/*
 * Copyright (c) 2021 okubo toshihiro
 *
 * Released under the MIT license.
 * see https://opensource.org/licenses/MIT
 *
 */
#include "Adv/AdvDocument.h"
#include "AdvWriter.hxx"

using namespace std;

int  buffer_delete_rc (char* buff)
{
    int i;
    int len;
    char*  str = buff;

    len = strlen(str);
    if(len > 0) {
        i = len - 1;
    } else {
        return -1;
    }

    if(str[i] == 0x0a) {
        str[i] = 0x00;
        i --;
    }
    if(str[i] == 0x0d) {
        str[i] = 0x00;
        i --;
    }

    for(; i > 0 && str[i] == 0x20; i --) {
        str[i] = 0x00;
    }

    return 0;
}

/////////////////////////////////////////
//  AdvWriter
/////////////////////////////////////////
AdvWriter
::AdvWriter()
{
  _fin = NULL;
  _adv = NULL;
  _doc = NULL;
}

AdvWriter
::~AdvWriter()
{

}

/*
  attribute type
  0 : string
  1 : int32
*/
int AdvWriter
::attributeType(string attriName)
{
  if(attriName == "content_type"
	 || attriName == "element_type"
	 || attriName == "format"
	 || attriName == "fega_type"
	 || attriName == "label")
	return 0;

  if(attriName == "num_items"
	 || attriName == "num_nodes_per_element"
	 || attriName == "dimension" 
	 || attriName == "index_byte")
	return 1;

  return 2;
}



bool AdvWriter
::read(char* inputFileName, char* outputFileName)
{

  //adv file open
  _adv  = adv_dio_file_open( outputFileName, "c" );

  if(_adv == 0)
  {
    fprintf(stderr,"Can not create %s\n",outputFileName);
    return false;
  }
  
    
  char buff[1024];
  
  _fin = fopen(inputFileName,"r");
  if(_fin == NULL){
    fprintf(stderr,"Can not find %s\n",inputFileName);
    return  false;
  }

  while (fgets(buff, 1024, _fin) != NULL){
    if(buff[0] == '#')
      continue;
    buffer_delete_rc (buff);

    if (strstr(buff, "[Properties]"))
    {
      this->writeProperty();
    }

  }

  adv_dio_file_close( _adv );
  return true;

}


bool AdvWriter
::writeProperty()
{
  
  _doc = adv_dio_create( _adv, adv_dio_make_documentid("Entity") );
  
  char  buff[1024];
  char  str[1024];
  char  keyValue[1024];

  string key;
  string value;

  
  while (fgets(buff, 1024, _fin) != NULL){
    if(buff[0] == '#')
       continue;
    buffer_delete_rc (buff);
    if(strlen(buff) == 0)
    {
      continue;
    }
	
    if (strstr(buff, "[Data]"))
    {
      this->writeData();	  
      adv_dio_close( _doc );
      return true;
    }

    sscanf(buff, "%s %s", str,keyValue);

	//create property
    char*  ptr = NULL;
    if((ptr = strchr(keyValue, '=')) == NULL)
    {
      fprintf(stderr,"[ERROR] property invalid\n");
      return false;
    }else{
      *ptr = '\0';
      key = keyValue;
      value = ptr + 1;

      int  type =  this->attributeType(key);
      if(type == 0)
       {
	    adv_dio_set_property( _doc, key.c_str(), value.c_str());
       }
	  
      if(type == 1)
      {
         adv_dio_set_property_int32( _doc, key.c_str(), atoi(value.c_str()) );
      }

       _keyValue[key] = value;

    }
	  
  }

  
  _keyValue.clear();
 
  return true;
}

bool  AdvWriter
::split(char*  buff,  const char* delim, vector<string>& returnContents)
{
  string src = buff;

  string::size_type len = src.length();

  for (string::size_type i = 0, n; i < len; i = n + 1) {
    n = src.find_first_of(delim, i);
    if (n == string::npos) {
      n = len;
    }
    returnContents.push_back(src.substr(i, n - i));
  }

  return  true;
}

bool  AdvWriter
::trim( char *s )
{
    int i, j;
 
    for( i = strlen(s)-1; i >= 0 && isspace( s[i] ); i-- ) ;
    s[i+1] = '\0';

    for( i = 0; isspace( s[i] ); i++ ) ;

    if( i > 0 ) {
        j = 0;
        while( s[i] ) s[j++] = s[i++];
        s[j] = '\0';
    }

	return  true;
}

bool AdvWriter
::writeData()
{
  char  buff[1024];
  vector<string>   contents;
  
  string num = _keyValue["num_items"];
  int numData = atoi(num.c_str());
  adv_off_t off = 0;

  string form = _keyValue["format"];
  int iForm = 0;

  if(_keyValue["fega_type"] == "NodeVariable"
	 || _keyValue["fega_type"] == "ElementVariable")
  {
	string index("i4");
	form = index + form;
  }

  
  for(int i = 0; i < numData; i++ )
  {
	if(fgets(buff,1024,_fin) == NULL)
	{
	  fprintf(stderr,"Can not read data\n");
	  return false;
	}
	
    buffer_delete_rc (buff);
	this->trim(buff);
	contents.clear();
	
    this->split(buff," ",contents);


	iForm = 0;
	for ( size_t iContents = 1; iContents < contents.size(); iContents++ ) {

	  if(form[iForm] == 'i')
	  {
        off += adv_dio_write_int32( _doc, off, atoi(contents[iContents].c_str()) );
      }
	  if(form[iForm] == 'f')
	  {
		off += adv_dio_write_float64( _doc, off, atof(contents[iContents].c_str()) );
      }

	  iForm = iForm + 2;
	}
  }


  return true;
}
